#ifndef PAINTING_H
#define PAINTING_H

#include "globals.h"
#include "operator_cost.h"
#include "utilities.h"

#include <set>
#include <string>
#include <vector>


enum PROFILE {
    FORK,
    I_FORK,
    X_SHAPE,
    INC_ARCS
};

class OptionParser;
class Options;

class Painting {
    
    bool incremental_search;
        
protected:
    
    bool force_cg_leaves_red;
    
    std::vector<bool> is_red_variable;
    
    virtual std::vector<bool> get_painting() = 0;
    
    std::vector<std::set<int> > get_sccs(std::vector<int> variables = std::vector<int>()) const;
    
    std::vector<std::set<int> > get_connected_components(std::vector<int> variables) const;
    
    std::vector<std::vector<std::set<int> > > get_scc_levels(std::vector<std::set<int> > sccs) const;
    
    std::vector<int> get_variable_levels() const;
    
    void apply_painting();
    
    void print_painting() const;
    
    void exit_error(std::string message, ExitCode exit_code) const;
    
    Painting(const Options &ops);
    
public:
    
    bool is_incremental_search() const {
        return incremental_search;
    }
    
    void do_painting();
    
    bool is_black_var(int var) const {
        return !is_red_variable[var];
    }
    
    bool is_red_var(int var) const {
        return is_red_variable[var];
    }
    
    static void add_options_to_parser(OptionParser &parser);
    
};

#endif
 
